VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "CConnectionEnd"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'Accessible UML

'LICENCE
'This program is free software: you can redistribute it and/or modify
'    it under the terms of the GNU General Public License as published by
'    the Free Software Foundation, either version 3 of the License, or
'    (at your option) any later version.
'
'    This program is distributed in the hope that it will be useful,
'    but WITHOUT ANY WARRANTY; without even the implied warranty of
'    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
'    GNU General Public License for more details.
'
'    You should have received a copy of the GNU General Public License
'    along with this program.  If not, see <http://www.gnu.org/licenses/>.

'"Accessible UML" is copyright 2001-2004 Paul Blenkhorn and Gareth Evans, at UMIST,
'and is released by them under the GNU Public Licence Version 3. A copy is available
'at http://www.gnu.org/licenses/gpl.txt
'The code was written by Alasdair King.

Option Explicit

Private cID As String ' a node ID indentifying this connectionEnd
Private cTarget As String ' the uniqueid of the target of this connectionEnd, ie another node
Private cOther As String ' the other connectionEnd in this relationship
Private cParent As CNode ' the parent node to which this belongs
'               parent                      targetNode
'   connectionEnd   CONNECTIONEND
'                <--other   target-->

Private cMultiplicity As String
Private cDirection As String
Private cNavigable As Boolean
Private cAggregation As String
Private cType As String
Private cName As String ' name of the connection

Public Property Get navigable() As Boolean

    On Error Resume Next

    navigable = cNavigable
End Property

Public Property Let navigable(newNavigable As Boolean)

    On Error Resume Next

    cNavigable = newNavigable
End Property

Public Property Get aggregation() As String

    On Error Resume Next

    aggregation = cAggregation
End Property

Public Property Let aggregation(newAggregation As String)

    On Error Resume Next

    cAggregation = newAggregation
End Property

Public Property Get info() As String

    On Error Resume Next

'returns information on this connection end
    info = Me.name & vbNewLine
    info = info & "Type: " & cType & vbNewLine
    If cDirection <> Empty Then
        info = info & "Direction: " & cDirection & vbNewLine
    End If
    If cMultiplicity <> Empty Then
        info = info & "Multiplicity: " & cMultiplicity & vbNewLine
    End If
    If cAggregation <> Empty Then
        info = info & "Aggregation: " & cAggregation
    End If

End Property

Public Property Get name() As String
    On Error Resume Next
    name = cName
End Property

Public Property Get nameFromParent() As String
'returns the name when this connection is viewed from the owning parent
    On Error Resume Next
    nameFromParent = cName
End Property

Public Property Get nameFromFarEnd() As String
'returns the name to be displayed when the far end connection wants to know what you are. For example, if this is
'one end of an association between class A and class B, then it resides in assocation X. If the user in class
'A wants to know the name of this connection, they do NOT want to see "Association with A". They want to see
'"Association with B". This is the name of the OTHER connectionEnd in the node, the one indicated by .other
    On Error Resume Next
    nameFromFarEnd = complete(other).name
End Property

Public Sub DetermineName()
    On Error Resume Next
'this determines what the human-friendly display name of the connection end should be. It is constructed from
'examination of the attributes of the connection itself
    Dim connectionIterator As CConnection
    Dim otherNode As CNode
    
    'since I am a connection end, I know that I have a parent object, which must be a relationship node
    '(well, not strictly true: if we have removed relationship nodes, it might be a proper node) and a
    'destination node, which must be a proper node.
    'My name should be something like
    '"Association relationship with Customer" or "[Type] relationship [direction] [destinationName]
    'Debug.Print "Connection End type = " & cType
    'I make an exception for use case diagrams, where we say "Used by" or "Uses"
    If complete(cTarget).nodeType = "UseCase" And cParent.nodeType = "Actor" Then
        '"used by" relationship
        cName = "Uses " & complete(cTarget).name
    ElseIf complete(cTarget).nodeType = "Actor" And cParent.nodeType = "UseCase" Then
        '"uses" relationship
        cName = "Used by " & complete(cTarget).name
    Else
        'not a use case diagram
        cName = cType & " "
        If cDirection = Empty Or cDirection = "both" Then
            cName = cName & "with"
        Else
            cName = cName & cDirection
        End If
        cName = cName & " " & complete(cTarget).name
        'Debug.Print "ConnectionEnd named: " & cName
    End If
End Sub

Public Property Get other() As String
    On Error Resume Next
    other = cOther
End Property

Public Property Let other(newOther As String)
    On Error Resume Next
    cOther = newOther
End Property

Public Property Get x() As Integer
    On Error Resume Next
    x = cParent.location.x
End Property

Public Property Get y() As Integer
    On Error Resume Next
    y = cParent.location.y
End Property

Public Property Get nodeID() As String
    On Error Resume Next
    nodeID = cParent.id
End Property

Public Property Get nodeName() As String
    On Error Resume Next
    nodeName = cParent.name
End Property

Public Property Get parent() As CNode
    On Error Resume Next
    Set parent = cParent
End Property

Public Property Set parent(newParent As CNode)
    On Error Resume Next
    Set cParent = newParent
End Property

Public Property Get isConnectionEnd() As Boolean
    On Error Resume Next
    isConnectionEnd = True
End Property

Public Property Let endType(newType As String)
    On Error Resume Next
    cType = newType
End Property

Public Property Get endType() As String
    On Error Resume Next
    endType = cType
End Property

Public Property Get id() As String
    On Error Resume Next
    id = cID
End Property

Public Property Let id(newID As String)
    On Error Resume Next
    cID = newID
End Property

Public Property Get target() As String
    On Error Resume Next
    target = cTarget
End Property

Public Property Let target(newTarget As String)
    On Error Resume Next
    cTarget = newTarget
End Property

Public Property Let multiplicity(newMultiplicity As String)
    On Error Resume Next
    cMultiplicity = newMultiplicity
End Property

Public Property Get multiplicity() As String
    On Error Resume Next
    multiplicity = cMultiplicity
End Property

Public Property Get direction() As String
    On Error Resume Next
    direction = cDirection
End Property

Public Property Let direction(newDirection As String)
    On Error Resume Next
    cDirection = newDirection
End Property
    
Public Sub RemoveConnectionsTo(otherNode As CNode)
'remove this connection end from this node
'(We know this end must be the connected item to the other node because it is
'identified by unique ID in the complete Collection.
    On Error Resume Next
    Call cParent.RemoveConnectionEnd(Me)
End Sub
