<?xml version="1.0" encoding="UTF-8"?>
<!-- This stylesheet converts XML files derived from UML XMI 1.2 files into application files that don't have connections represented as nodes -->
<!-- Jan 2004 -->
<xsl:stylesheet
      xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
      version="1.0">
	<xsl:output method="xml" omit-xml-declaration = "no" indent="yes" encoding="UTF-8"/>
	<!-- Set-up an indexed table of the nodes in the file referenced by the unique xmi.id attribute -->
	<!-- Allows use of the efficient key() function -->
	<xsl:key name="nodeIndex" match="*[@id]" use="@id" />

	
	<xsl:template match="/">
		<!-- Add the TeDUB DTD reference -->
		<!--<xsl:text disable-output-escaping="yes">
			<![CDATA[
    			<!DOCTYPE diagram SYSTEM "tedub.dtd">
			]]>	
    	</xsl:text> -->
		<!-- Create the diagram -->
       	<diagram>
       		<xsl:attribute name="languageVersion"><xsl:value-of select="@languageVersion"/></xsl:attribute>
       		<xsl:attribute name="type"><xsl:value-of select="@type"/></xsl:attribute>
       		<xsl:apply-templates/>	
		</diagram>
   	</xsl:template>
   	<xsl:template match="node">
   		<!-- Have to do an exhaustive check of types of node, and only include those we want -->
   		<!-- <xsl:if test="@type = 'Class' or @type='Model' or @type='Interface' or @type='StateMachine' or @type='State machine diagram' or @type='CompositeState' or @type='Pseudostate' or @type='FinalState' or @type='SimpleState' or @type='Object' or @type='Actor' or @type='UseCase'">  -->
   		<xsl:if test="@type != 'Association' and @type != 'Transition' and @type != 'Dependency' and @type != 'Extend' and @type != 'Include' and @type != 'Generalization'">
   			<node>
   				<!-- attributes of main node -->
   				<xsl:attribute name="type"><xsl:value-of select="@type"/></xsl:attribute>
   				<xsl:attribute name="name"><xsl:value-of select="@name"/></xsl:attribute>
   				<xsl:attribute name="id"><xsl:value-of select="@id"/></xsl:attribute>
   				<xsl:attribute name="number"><xsl:value-of select="@number"/></xsl:attribute>
   				<xsl:for-each select="note"><note><xsl:value-of select="current()"/></note></xsl:for-each>
   				<!-- attribute nodes of main node -->
   				<xsl:for-each select="attribute">
   					<attribute>
   						<xsl:attribute name="type"><xsl:value-of select="@type"/></xsl:attribute>
   						<xsl:attribute name="name"><xsl:value-of select="@name"/></xsl:attribute>
   						<xsl:attribute name="visibility"><xsl:value-of select="@visibility"/></xsl:attribute>
   						<xsl:attribute name="value"><xsl:value-of select="@value"/></xsl:attribute>
   						<xsl:for-each select="parameter">
   							<parameter>
   								<xsl:attribute name="name"><xsl:value-of select="@name"/></xsl:attribute>
   								<xsl:attribute name="kind"><xsl:value-of select="@kind"/></xsl:attribute>
   								<xsl:attribute name="type"><xsl:value-of select="@type"/></xsl:attribute>
   							</parameter>
   						</xsl:for-each>
   					</attribute>
   				</xsl:for-each>
   				<!-- connections: need to get each one, and correct them to connectionEnds if they are connections to associations -->
				<xsl:variable name="uniqueID" select="@id"/>
   				<xsl:for-each select="connection">
   					<!-- got a connection: decide whether to add it -->
   					<xsl:choose>
   						<xsl:when test="//*[@id = current()/@href and local-name() = 'connectionEnd']">
   							<!-- this connection links to a connection end, and must be replaced by that connection end -->
   							<xsl:for-each select="//*[@id= current()/@href]">
   								<!-- CONTEXT: connectionEnd to be copied -->
   								<connectionEnd id="{@id}" navigable="{@navigable}" direction="{@direction}" aggregation="{@aggregation}" multiplicity="{@multiplicity}">
									<!-- the href (target) is the tricky bit. We want the target of the OTHER end of the the node owning the two connectionNodes... -->
									<xsl:attribute name="href">
										<xsl:value-of select="parent::*/connectionEnd[@id != current()/@id]/@href"/>
									</xsl:attribute>
   								</connectionEnd>
   							</xsl:for-each>
   						</xsl:when>
   						<xsl:when test="//*[@id = current()/@href and @type = 'Transition']">
   							<!-- this is a connection to a transition, which has to be cut out -->
   							<!-- get the end pointing away from us -->
   							<xsl:for-each select="//*[@id = current()/@href and @type = 'Transition']">
   								<!-- this is the transition node -->
   								<xsl:for-each select="connection[@href != $uniqueID]">
	  								<connection>
	   									<xsl:attribute name="relationship"><xsl:value-of select="@relationship"/></xsl:attribute>
			   							<xsl:attribute name="href"><xsl:value-of select="@href"/></xsl:attribute>
	   								</connection>
   								</xsl:for-each>
   							</xsl:for-each>
   						</xsl:when>
						<xsl:when test="//*[@id = current()/@href and @type = 'Dependency']">
   							<!-- this is a connection to a dependency, which has to be cut out -->
   							<!-- get the end pointing away from us -->
   							<xsl:for-each select="//*[@id = current()/@href and @type = 'Dependency']">
   								<!-- this is the transition node -->
   								<xsl:for-each select="connection[@href != $uniqueID]">
	  								<connection>
	   									<xsl:attribute name="relationship"><xsl:value-of select="@relationship"/></xsl:attribute>
			   							<xsl:attribute name="href"><xsl:value-of select="@href"/></xsl:attribute>
	   								</connection>
   								</xsl:for-each>
   							</xsl:for-each>
   						</xsl:when>
   						<xsl:when test="//*[@id = current()/@href and @type = 'Extend']">
   							<!-- this is a connection to a Extend node, which has to be cut out -->
   							<!-- get the end pointing away from us -->
   							<xsl:for-each select="//*[@id = current()/@href and @type = 'Extend']">
   								<!-- this is the transition node -->
   								<xsl:for-each select="connection[@href != $uniqueID]">
	  								<connection>
	   									<xsl:attribute name="relationship"><xsl:value-of select="@relationship"/></xsl:attribute>
			   							<xsl:attribute name="href"><xsl:value-of select="@href"/></xsl:attribute>
	   								</connection>
   								</xsl:for-each>
   							</xsl:for-each>
   						</xsl:when>
   						<xsl:when test="//*[@id = current()/@href and @type = 'Include']">
   							<!-- this is a connection to a Include node, which has to be cut out -->
   							<!-- get the end pointing away from us -->
   							<xsl:for-each select="//*[@id = current()/@href and @type = 'Include']">
   								<!-- this is the transition node -->
   								<xsl:for-each select="connection[@href != $uniqueID]">
	  								<connection>
	   									<xsl:attribute name="relationship"><xsl:value-of select="@relationship"/></xsl:attribute>
			   							<xsl:attribute name="href"><xsl:value-of select="@href"/></xsl:attribute>
	   								</connection>
   								</xsl:for-each>
   							</xsl:for-each>
   						</xsl:when>
   						<xsl:when test="//*[@id = current()/@href and @type = 'Generalization']">
   							<!-- this is a connection to a Generalization node, which has to be cut out -->
   							<!-- get the end pointing away from us -->
   							<xsl:for-each select="//*[@id = current()/@href and @type = 'Generalization']">
   								<!-- this is the transition node -->
   								<xsl:for-each select="connection[@href != $uniqueID]">
	  								<connection>
	   									<xsl:attribute name="relationship"><xsl:value-of select="@relationship"/></xsl:attribute>
			   							<xsl:attribute name="href"><xsl:value-of select="@href"/></xsl:attribute>
	   								</connection>
   								</xsl:for-each>
   							</xsl:for-each>
   						</xsl:when><xsl:otherwise>
   							<!-- this connection simply links to a proper node, and will continue to do so. Copy it in unaltered -->
		   					<connection>
		   						<xsl:attribute name="relationship"><xsl:value-of select="@relationship"/></xsl:attribute>
		   						<xsl:attribute name="href"><xsl:value-of select="@href"/></xsl:attribute>
		   					</connection>
		   				</xsl:otherwise>
		   			</xsl:choose>
   				</xsl:for-each>
   				<xsl:if test="location">
					<location>
						<xsl:attribute name="x"><xsl:value-of select="location/@x"/></xsl:attribute>
						<xsl:attribute name="y"><xsl:value-of select="location/@y"/></xsl:attribute>
					</location>
   				</xsl:if>
   				<xsl:for-each select="connectionEnd">
   					<xsl:copy-of select="."/>
   				</xsl:for-each>
   				<!-- Do child nodes -->
		   	  	<xsl:apply-templates/>	
			</node>
   		</xsl:if>
	</xsl:template>
	<xsl:template match="comment()"><xsl:comment><xsl:value-of select="self::comment()"/></xsl:comment></xsl:template>
	<xsl:template match="//connection"/>
	<xsl:template match="//connectionEnd"/>
	<xsl:template match="//note"/>
</xsl:stylesheet>