Attribute VB_Name = "modLoader"
'Accessible UML

'LICENCE
'This program is free software: you can redistribute it and/or modify
'    it under the terms of the GNU General Public License as published by
'    the Free Software Foundation, either version 3 of the License, or
'    (at your option) any later version.
'
'    This program is distributed in the hope that it will be useful,
'    but WITHOUT ANY WARRANTY; without even the implied warranty of
'    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
'    GNU General Public License for more details.
'
'    You should have received a copy of the GNU General Public License
'    along with this program.  If not, see <http://www.gnu.org/licenses/>.

'"Accessible UML" is copyright 2001-2004 Paul Blenkhorn and Gareth Evans, at UMIST,
'and is released by them under the GNU Public Licence Version 3. A copy is available
'at http://www.gnu.org/licenses/gpl.txt
'The code was written by Alasdair King.

'Contains code for converting the XML document into the internal data representation

Option Explicit

Public Function ParseDocument(document As DOMDocument30) As CNode
    On Error Resume Next
    'returns the top node of the diagram containing the whole document/model
    Dim topIterator As IXMLDOMNode
    Dim topNode As CNode
    Dim nodeIterator As CNode
    Dim bookmarkNode As IXMLDOMNodeList
'    Dim nodeChildIterator As Object
    Dim result As CNode
    Dim connectionIterator As CConnection
    Dim completeIterator As Object
    Dim x As Integer
    Dim y As Integer
    Dim a As Integer
    Dim b As Integer
    Dim i As Integer
    
    Set result = ParseNode(document.documentElement.selectNodes("node").item(0))
    're-assign complete to make it complete
    Set complete = result.allNodes
    'now do the order of nodes within diagrams
    For Each nodeIterator In diagramNodes
        If nodeIterator.isDiagram Then
            Call modNodeSorter.SortDiagram(nodeIterator)
        End If
    Next nodeIterator
    'do layout of sequence nodes
    For Each nodeIterator In diagramNodes
        If nodeIterator.nodeType = "Collaboration" Then
            'This is a sequence node: have to lay it out specially
            Call nodeIterator.LayoutSequenceDiagramNodes
        End If
    Next nodeIterator
    'do connections of contents of sequence diagram nodes (that is, the objects
    'that make up the diagram) to the messages relating to that node
    For Each nodeIterator In diagramNodes
        If nodeIterator.nodeType = "Sequence Diagram" Then
            Call AddMessagesToSequenceDiagram(nodeIterator)
        End If
    Next nodeIterator
'    Call ConstructSequenceDiagram(coordinateMap)
    'now do naming of nodes and connections: first, do nodes
    For Each nodeIterator In diagramNodes
        If nodeIterator.connectionEnds.Count > 0 Then
            Call nodeIterator.DetermineName
        End If
    Next nodeIterator
    'now do connectionEnd nodes
    For Each completeIterator In complete
        If Len(completeIterator.name) = 0 Then
            Call completeIterator.DetermineName
        End If
    Next completeIterator
   'now do connections
    For Each nodeIterator In diagramNodes
        For Each connectionIterator In nodeIterator.connections
            Call connectionIterator.DetermineName(nodeIterator)
        Next connectionIterator
    Next nodeIterator
    'now do directions between connected nodes
    For Each nodeIterator In diagramNodes
        If nodeIterator.location.placed Then
            'placed on diagram: work out directions
            Call nodeIterator.CalculateRelationshipDirections(result.allNodes)
            Call nodeIterator.CalculateCompassDirections(result.allNodes)
'        Else
'            'we're not placed on the diagram, but still locate the joystick
'            Call nodeIterator.CalculateCentre
        End If
    Next nodeIterator
    'bookmarks
    If document.documentElement.selectNodes("bookmarks").length > 0 Then
        'check for bookmark nodes
        Set bookmarkNode = document.documentElement.selectSingleNode("bookmarks").selectNodes("bookmark")
        For i = 1 To bookmarkNode.length
            If i > 1 Then
                Call Load(frmMain.mnuBookmarkList(i - 1))
            End If
            frmMain.mnuBookmarkList(i - 1).Visible = True
            frmMain.mnuBookmarkList(i - 1).Caption = "&" & i & " " & bookmarkNode.item(i - 1).attributes.getNamedItem("name").Text
            Call bookmarks.add(CStr(i), bookmarkNode.item(i - 1).attributes.getNamedItem("target").Text)
        Next i
    End If
    'return
    Set ParseDocument = result
End Function

Private Function ParseNode(aNode As IXMLDOMNode) As CNode
    On Error Resume Next
'parses aNode into a CNode
    Dim newNode As New CNode
    Call newNode.ParseXML(aNode)
    Set ParseNode = newNode
End Function

''Private Sub ConstructSequenceDiagram(coordinateMap As Dictionary)
'''since sequence diagrams are presented radically differently, they need an extra stage of processing
'''coordinateMap is a completed dictionary containing object name:x coord pairs
''    On Error Resume Next
''    Dim nodeIterator As CNode
''    Dim completeIterator As Object
''    Dim sequenceNodeIterator As CNode
''    Dim xPositions As String
''    Dim yPositions As String
''    Dim objectName As String
''    Dim found As Boolean
''    Dim newDoc As MSXML2.DOMDocument30
''    Dim newNode As CNode
''    Dim reverseCoordinateMap As New Dictionary
''    Dim i As Integer
''    Dim splitX() As String
''    Dim splitY() As String
''    Dim nX As Integer
''    Dim nY As Integer
''    Dim iX As Integer
''    Dim iY As Integer
''    Dim moved As Boolean
''    Dim yCounter As Integer
''    Dim yTarget As Integer
''    Dim bestY As Integer
''    Dim nRows As Integer
''    Dim nColumns As Integer
''    Dim rowIterator As Integer
''    Dim columnIterator As Integer
''    Dim newConnection As CConnection
''    Dim nodeByLocationIterator As CNode
''
''    'before we start, reverse coordinateMap to relate positions to object names
''    For i = 0 To coordinateMap.Count - 1
''        Call reverseCoordinateMap.add(coordinateMap.Items(i), coordinateMap.Keys(i))
''    Next i
''
''    'first, test if we have a sequence diagram
''    For Each completeIterator In complete
''        'Debug.Print "nodeIterator is " & nodeIterator.nodeType & " " & nodeIterator.nodeName
''        If completeIterator.isConnectionEnd Then
''            'whoops, this is a connection end: move along
''        Else
''            'got a node
''            Set nodeIterator = completeIterator
''            If nodeIterator.nodeType = "Collaboration" Then
''                'found a sequence diagram!
''                'right, iterate through the existing nodes and work out all the places that there should be a node
''                For Each sequenceNodeIterator In nodeIterator.childNodes
''                    'new position to record
''                    If InStr(1, xPositions, CStr(sequenceNodeIterator.location.x)) = 0 Then
''                        'not recorded this x yet: do so now
''                        xPositions = xPositions & " " & CStr(sequenceNodeIterator.location.x)
''                    End If
''                    If InStr(1, yPositions, CStr(sequenceNodeIterator.location.y)) = 0 Then
''                        'not recorded this y yet: do so now
''                        yPositions = yPositions & " " & CStr(sequenceNodeIterator.location.y)
''                    End If
''                Next sequenceNodeIterator
''                'assertion: xPositions holds all of the x coordinates that should contain a sequence node
''                'assertion: yPositions holds all of the y coordinates that should contain a sequence node
''                splitX = Split(Trim(xPositions), " ")
''                nX = UBound(splitX)
''                splitY = Split(Trim(yPositions), " ")
''                nY = UBound(splitY)
''
''                'now go through all the x and ys checking nodes are there: if not found, create them
''                For iX = 0 To nX
''                    For iY = 0 To nY
''                        'okay, we have an x,y: is there a node here?
''                        found = False
''                        For Each sequenceNodeIterator In nodeIterator.childNodes
''                            If sequenceNodeIterator.location.x = CInt(splitX(iX)) And sequenceNodeIterator.location.y = CInt(splitY(iY)) Then
''                                'goody, already here: found it
''                                found = True
''                                Exit For
''                            End If
''                        Next sequenceNodeIterator
''                        'did we find a sequence node where one should be?
''                        If Not found Then
''                            'nope, no node: have to create one here.
''                            Set newDoc = New MSXML2.DOMDocument30
''                            Call newDoc.loadXML("<node type=""Inactive object"" id=""inactiveobject" & CStr(splitX(iX)) & CStr(splitY(iY)) & """ name=""" & reverseCoordinateMap.item(CInt(splitX(iX))) & """><location x=""" & CStr(splitX(iX)) & """ y=""" & CStr(splitY(iY)) & """/></node>")
''                            'Debug.Print newDoc.xml
''                            Set newNode = New CNode
''                            Call newNode.ParseXML(newDoc.documentElement)
''                            Set newNode.parent = nodeIterator
''                            Call nodeIterator.AddChild(newNode)
''                            Call complete.add(newNode, newNode.id)
''                        End If
''                    Next iY
''                Next iX
''                'now move all the nodes so that the interval between them is one unit, starting at 1
''                moved = True
''                yCounter = 1 ' where we're currently shifting to
''                While moved 'repeat until we move nothing
''                    moved = False
''                    For Each sequenceNodeIterator In nodeIterator.childNodes
''                        'we can tell if we have not yet finished because we know that when we have finished the node rows
''                        'should be stacked up 1, 2, 3... nY+1. So any above nY +1 indicate that we haven't finished yet.
''                        'Debug.Print "Looking at node at " & sequenceNodeIterator.location.Y
''                        If sequenceNodeIterator.location.y > nY + 1 Then
''                            'Debug.Print "Triggered moved: will replace"
''                            moved = True
''                            Exit For
''                        End If
''                    Next sequenceNodeIterator
''                    'finished checking for movement: moved holds the result
''                    If moved Then
''                        'nope, got a node to move. Go through the nodes looking for node closest to ycounter -1 and move it
''                        bestY = 32000
''                        For Each sequenceNodeIterator In nodeIterator.childNodes
''                            If sequenceNodeIterator.location.y > yCounter - 1 Then
''                                'contender for movement
''                                If sequenceNodeIterator.location.y < bestY Then
''                                    'best contender so far
''                                    bestY = sequenceNodeIterator.location.y
''                                End If
''                            End If
''                        Next sequenceNodeIterator
''                        'assertion: bestY should hold the y coord of the nodes to move to yCounter
''                        'Debug.Print "Nodes to move are those at " & bestY
''                        'now move the nodes: find and shift, keeping x as it was
''                        For Each sequenceNodeIterator In nodeIterator.childNodes
''                            If sequenceNodeIterator.location.y = bestY Then
''                                'found a node to shift
''                                Call sequenceNodeIterator.location.Move(sequenceNodeIterator.location.x, yCounter)
''                                'Debug.Print "Shifted node from " & bestY
''                            End If
''                        Next sequenceNodeIterator
''                        'assertion: shifted all the nodes: increment yCounter
''                        yCounter = yCounter + 1
''                        'Debug.Print "yCounter now " & yCounter
''                    Else
''                        'moved is false: we've finished moving nodes: we'll exit the while loop
''                    End If
''                Wend
''                'The next thing to do is to connect all the nodes in chronological order
''                'We know that we have nX+1 columns and nY+1 rows
''                nRows = nY + 1
''                nColumns = nX + 1
''                'go through all the nodes in their positions and connect them up
''                For rowIterator = 1 To nRows
''                    For columnIterator = 1 To nColumns
''                        'we are at position (columnIterator, rowIterator). Find the node there
''                        For Each sequenceNodeIterator In nodeIterator.childNodes
''                            'is this the node?
''                            If sequenceNodeIterator.location.x = columnIterator And sequenceNodeIterator.location.y = rowIterator Then
''                                'found the node
''                                If rowIterator > 1 Then
''                                    'do a connection to the node above: have to find it first!
''                                    For Each nodeByLocationIterator In nodeIterator.childNodes
''                                        If nodeByLocationIterator.location.x = columnIterator And nodeByLocationIterator.location.y = rowIterator - 1 Then
''                                            'found other node
''                                            Set newConnection = New CConnection
''                                            newConnection.target = nodeByLocationIterator.id
''                                            newConnection.id = nodeByLocationIterator.id & "-" & GetTickCount
''                                            newConnection.connectionType = frmMain.language.GetText("Previous moment in time")
''                                            Call sequenceNodeIterator.AddConnection(newConnection)
''                                            Exit For
''                                        End If
''                                    Next nodeByLocationIterator
''                                End If
''                                If rowIterator < nRows Then
''                                    'do a connection to the node below
''                                    For Each nodeByLocationIterator In nodeIterator.childNodes
''                                        If nodeByLocationIterator.location.x = columnIterator And nodeByLocationIterator.location.y = rowIterator + 1 Then
''                                            'found other node
''                                            Set newConnection = New CConnection
''                                            newConnection.target = nodeByLocationIterator.id
''                                            newConnection.id = nodeByLocationIterator.id & "-" & GetTickCount
''                                            newConnection.connectionType = frmMain.language.GetText("Next moment in time")
''                                            Call sequenceNodeIterator.AddConnection(newConnection)
''                                            Exit For
''                                        End If
''                                    Next nodeByLocationIterator
''                                End If
''                                Exit For
''                            End If
''                        Next sequenceNodeIterator
''                    Next columnIterator
''                Next rowIterator
''                'Debug.Print "We have " & nRows; " rows and " & nColumns & " columns."
''                'finally, rename the nodes by time (y coordinate)
''                For Each sequenceNodeIterator In nodeIterator.childNodes
''                    sequenceNodeIterator.name = sequenceNodeIterator.name & " " & frmMain.language.GetText("at time") & " " & sequenceNodeIterator.location.y
''                Next sequenceNodeIterator
''            End If
''        End If
''    Next completeIterator
''End Sub
