Attribute VB_Name = "modSaver"
'Accessible UML

'LICENCE
'This program is free software: you can redistribute it and/or modify
'    it under the terms of the GNU General Public License as published by
'    the Free Software Foundation, either version 3 of the License, or
'    (at your option) any later version.
'
'    This program is distributed in the hope that it will be useful,
'    but WITHOUT ANY WARRANTY; without even the implied warranty of
'    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
'    GNU General Public License for more details.
'
'    You should have received a copy of the GNU General Public License
'    along with this program.  If not, see <http://www.gnu.org/licenses/>.

'"Accessible UML" is copyright 2001-2004 Paul Blenkhorn and Gareth Evans, at UMIST,
'and is released by them under the GNU Public Licence Version 3. A copy is available
'at http://www.gnu.org/licenses/gpl.txt
'The code was written by Alasdair King.

Option Explicit

Private diagramSave As DOMDocument30
Private exportingForSVG As Boolean

Public Sub SaveForSVG(diagramNode As CNode)
'saves current diagram node to disk for SVG conversion
    On Error Resume Next
    Dim newXMLAttribute As IXMLDOMAttribute
    Dim newNode As IXMLDOMNode
    Dim otherNode As CNode
    Dim exitNode As CExit
       
    'record whether this is for SVG export: if so, we save more friendly data
    exportingForSVG = True
    'file to hold diagram
    Set diagramSave = New DOMDocument30
    'setup document with first top node in diagram
    Call diagramSave.loadXML("<diagram></diagram>")
    'add diagram attributes
    Set newXMLAttribute = diagramSave.createNode(NODE_ATTRIBUTE, "languageVersion", Empty)
    newXMLAttribute.Text = "1000" ' this indicates a saved (as opposed to imported/transformed) file
    Call diagramSave.documentElement.attributes.setNamedItem(newXMLAttribute)
    Set newXMLAttribute = diagramSave.createNode(NODE_ATTRIBUTE, "type", Empty)
    Select Case diagramType
        Case BOX_AND_POINTER_DIAGRAM
            newXMLAttribute.Text = DIAGRAM_BOX_AND_POINTER
        Case UML_DIAGRAM
            newXMLAttribute.Text = DIAGRAM_UML
        Case Else
            newXMLAttribute.Text = "Generic Diagram"
    End Select
    Call diagramSave.documentElement.attributes.setNamedItem(newXMLAttribute)
    'build diagram
    For Each exitNode In currentNode.exits
        Set otherNode = complete(exitNode.target)
        If complete(exitNode.target).isConnectionEnd Then
            Set otherNode = complete(exitNode.target).parent
        Else
            Set otherNode = complete(exitNode.target)
        End If
        'add this node to the SVG
        Call diagramSave.documentElement.appendChild(ParseNode(otherNode))
    Next exitNode
    'save to disk
    Call diagramSave.save(App.path & "\temp\toSVG.xml")
End Sub

Public Sub SaveDiagram(path As String, Optional fromNode As CNode, Optional maxDepth As Integer, Optional svg As Boolean)
'saves the diagram to disk, starting at fromNode: if fromNode is nothing, use the top of the diagram tree. Only go
'maxDepth nodes into the structure (so depth must be 1, creating the current node: otherwise 100 is used)
    On Error Resume Next
    Dim prettyStylesheet As New DOMDocument30
    Dim newXMLAttribute As IXMLDOMAttribute
    Dim newNode As IXMLDOMNode
    Dim newBookmark As IXMLDOMNode
    Dim i As Integer
       
    'record whether this is for SVG export: if so, we save more friendly data
    exportingForSVG = svg
    'create a fromNode
    If fromNode Is Nothing Then
        Set fromNode = diagramNode
    End If
    'check value for depth: if not given, use 32000 nodes
    If maxDepth <= 0 Then maxDepth = 32000
    'file to hold diagram
    Set diagramSave = New DOMDocument30
    'setup document with first top node in diagram
    Call diagramSave.loadXML("<diagram></diagram>")
    'add diagram attributes
    Set newXMLAttribute = diagramSave.createNode(NODE_ATTRIBUTE, "languageVersion", Empty)
    newXMLAttribute.Text = "1000" ' this indicates a saved (as opposed to imported/transformed) file
    Call diagramSave.documentElement.attributes.setNamedItem(newXMLAttribute)
    Set newXMLAttribute = diagramSave.createNode(NODE_ATTRIBUTE, "type", Empty)
    Select Case diagramType
        Case UML_DIAGRAM
            newXMLAttribute.Text = DIAGRAM_UML
        Case Else
            newXMLAttribute.Text = "Generic Diagram"
    End Select
    Call diagramSave.documentElement.attributes.setNamedItem(newXMLAttribute)
    'build diagram iteratively
    Call BuildDiagram(diagramSave.firstChild, fromNode, 1, maxDepth)
    'add bookmarks
    Set newNode = diagramSave.createNode(NODE_ELEMENT, "bookmarks", Empty)
    If frmMain.mnuBookmarkList(0).Visible = False Then
        'no bookmarks: add empty node
    Else
        For i = 0 To frmMain.mnuBookmarkList.Count - 1
            Set newBookmark = diagramSave.createNode(NODE_ELEMENT, "bookmark", Empty)
            'name
            Set newXMLAttribute = diagramSave.createNode(NODE_ATTRIBUTE, "name", Empty)
            newXMLAttribute.Text = Right(frmMain.mnuBookmarkList(i).Caption, Len(frmMain.mnuBookmarkList(i).Caption) - InStr(1, frmMain.mnuBookmarkList(i).Caption, " ", vbBinaryCompare))
            Call newBookmark.attributes.setNamedItem(newXMLAttribute)
            'target
            Set newXMLAttribute = diagramSave.createNode(NODE_ATTRIBUTE, "target", Empty)
            newXMLAttribute.Text = bookmarks(CStr(i))
            Call newBookmark.attributes.setNamedItem(newXMLAttribute)
            'add to bookmarks
            Call newNode.appendChild(newBookmark)
        Next i
    End If
    'add to diagram
    Call diagramSave.documentElement.appendChild(newNode)
    'format prettily
'    Call prettyStylesheet.Load(App.path & "\config\prettyFormat.xsl")
'    Call diagramSave.loadXML(diagramSave.transformNode(prettyStylesheet))
    'save to disk
    Call diagramSave.save(path)
End Sub

Private Function ParseNode(buildWith As CNode) As IXMLDOMNode
    On Error Resume Next
'parses a UML node into an XML node
    Dim newXMLNode As IXMLDOMNode
    Dim locationNode As IXMLDOMNode
    Dim newXMLAttribute As IXMLDOMAttribute
    Dim connectionIterator As CConnection
    Dim connectionEndIterator As CConnectionEnd
    Dim connectionNode As IXMLDOMNode
    Dim connectionEndNode As IXMLDOMNode
    Dim childIterator As CNode
    Dim polylineNode As IXMLDOMNode
    Dim pointNode As IXMLDOMNode
    Dim pointIterator As CPoint
    Dim noteNode As IXMLDOMNode
    Dim attributeIterator As CAttribute
    Dim attributeNode As IXMLDOMNode
    Dim parameterIterator As CParameter
    Dim parameterNode As IXMLDOMNode
    Dim noteSections() As String
    Dim i As Integer
    Dim textNode As IXMLDOMNode
    Dim breakNode As IXMLDOMNode
    
    Set newXMLNode = diagramSave.createNode(NODE_ELEMENT, "node", Empty)
    'do my attributes
    'id
    Set newXMLAttribute = diagramSave.createNode(NODE_ATTRIBUTE, "id", Empty)
    newXMLAttribute.Text = buildWith.id
    Call newXMLNode.attributes.setNamedItem(newXMLAttribute)
    'name
    Set newXMLAttribute = diagramSave.createNode(NODE_ATTRIBUTE, "name", Empty)
    If exportingForSVG Then
        newXMLAttribute.Text = buildWith.displayName
    Else
        newXMLAttribute.Text = buildWith.name
    End If
    Call newXMLNode.attributes.setNamedItem(newXMLAttribute)
    'type
    Set newXMLAttribute = diagramSave.createNode(NODE_ATTRIBUTE, "type", Empty)
    newXMLAttribute.Text = buildWith.nodeType
    Call newXMLNode.attributes.setNamedItem(newXMLAttribute)
    'annotation
    If Len(Trim(buildWith.annotation)) > 0 Then
        'annotation to save
        Set noteNode = diagramSave.createNode(NODE_ELEMENT, "note", Empty)
        noteSections = Split(buildWith.annotation, vbNewLine)
        For i = LBound(noteSections) To UBound(noteSections)
            Set textNode = diagramSave.createTextNode(noteSections(i))
            Call noteNode.appendChild(textNode)
            If i < UBound(noteSections) Then
                Set breakNode = diagramSave.createNode(NODE_ELEMENT, "br", Empty)
                Call noteNode.appendChild(breakNode)
            End If
        Next i
        Call newXMLNode.appendChild(noteNode)
    End If
    'add location child
    If buildWith.location.placed Then
        Set locationNode = diagramSave.createNode(NODE_ELEMENT, "location", Empty)
        Set newXMLAttribute = diagramSave.createNode(NODE_ATTRIBUTE, "x", Empty)
        newXMLAttribute.Text = buildWith.location.x
        Call locationNode.attributes.setNamedItem(newXMLAttribute)
        Set newXMLAttribute = diagramSave.createNode(NODE_ATTRIBUTE, "y", Empty)
        newXMLAttribute.Text = buildWith.location.y
        Call locationNode.attributes.setNamedItem(newXMLAttribute)
        'add to node
        Call newXMLNode.appendChild(locationNode)
    End If
    'add polyline child
    If buildWith.points.Count > 0 Then
        Set polylineNode = diagramSave.createNode(NODE_ELEMENT, "polyline", Empty)
        For Each pointIterator In buildWith.points
            Set pointNode = diagramSave.createNode(NODE_ELEMENT, "point", Empty)
            Set newXMLAttribute = diagramSave.createNode(NODE_ATTRIBUTE, "x", Empty)
            newXMLAttribute.Text = pointIterator.x
            Call pointNode.attributes.setNamedItem(newXMLAttribute)
            Set newXMLAttribute = diagramSave.createNode(NODE_ATTRIBUTE, "y", Empty)
            newXMLAttribute.Text = pointIterator.y
            Call pointNode.attributes.setNamedItem(newXMLAttribute)
            Call polylineNode.appendChild(pointNode)
        Next pointIterator
        'add to node
        Call newXMLNode.appendChild(polylineNode)
    End If
    'add connections
    For Each connectionIterator In buildWith.connections
        'check there is no connectionEnd
        Set connectionNode = diagramSave.createNode(NODE_ELEMENT, "connection", Empty)
        'href
        Set newXMLAttribute = diagramSave.createNode(NODE_ATTRIBUTE, "href", Empty)
        newXMLAttribute.Text = connectionIterator.target
        Call connectionNode.attributes.setNamedItem(newXMLAttribute)
        'type
        Set newXMLAttribute = diagramSave.createNode(NODE_ATTRIBUTE, "relationship", Empty)
        If exportingForSVG Then
            newXMLAttribute.Text = connectionIterator.name
        Else
            newXMLAttribute.Text = connectionIterator.connectionType
        End If
        Call connectionNode.attributes.setNamedItem(newXMLAttribute)
        'add to node
        Call newXMLNode.appendChild(connectionNode)
    Next connectionIterator
    'add connectionEnds
    For Each connectionEndIterator In buildWith.connectionEnds
        Set connectionEndNode = diagramSave.createNode(NODE_ELEMENT, "connectionEnd", Empty)
        'id
        Set newXMLAttribute = diagramSave.createNode(NODE_ATTRIBUTE, "id", Empty)
        newXMLAttribute.Text = connectionEndIterator.id
        Call connectionEndNode.attributes.setNamedItem(newXMLAttribute)
        'href
        Set newXMLAttribute = diagramSave.createNode(NODE_ATTRIBUTE, "href", Empty)
        newXMLAttribute.Text = connectionEndIterator.target
        Call connectionEndNode.attributes.setNamedItem(newXMLAttribute)
        'relationship
        Set newXMLAttribute = diagramSave.createNode(NODE_ATTRIBUTE, "relationship", Empty)
        newXMLAttribute.Text = connectionEndIterator.endType
        Call connectionEndNode.attributes.setNamedItem(newXMLAttribute)
        'direction
        Set newXMLAttribute = diagramSave.createNode(NODE_ATTRIBUTE, "direction", Empty)
        newXMLAttribute.Text = connectionEndIterator.direction
        Call connectionEndNode.attributes.setNamedItem(newXMLAttribute)
        'multiplicity
        Set newXMLAttribute = diagramSave.createNode(NODE_ATTRIBUTE, "multiplicity", Empty)
        newXMLAttribute.Text = connectionEndIterator.multiplicity
        Call connectionEndNode.attributes.setNamedItem(newXMLAttribute)
        'aggregation
        Set newXMLAttribute = diagramSave.createNode(NODE_ATTRIBUTE, "aggregation", Empty)
        newXMLAttribute.Text = connectionEndIterator.aggregation
        Call connectionEndNode.attributes.setNamedItem(newXMLAttribute)
        'navigable
        Set newXMLAttribute = diagramSave.createNode(NODE_ATTRIBUTE, "navigable", Empty)
        newXMLAttribute.Text = connectionEndIterator.navigable
        Call connectionEndNode.attributes.setNamedItem(newXMLAttribute)
        'add to parent
        Call newXMLNode.appendChild(connectionEndNode)
    Next connectionEndIterator
    'do attributes and operations
    For Each attributeIterator In buildWith.attributes
        Set attributeNode = diagramSave.createNode(NODE_ELEMENT, "attribute", Empty)
        'name
        Set newXMLAttribute = diagramSave.createNode(NODE_ATTRIBUTE, "name", Empty)
        newXMLAttribute.Text = attributeIterator.name
        Call attributeNode.attributes.setNamedItem(newXMLAttribute)
        'type
        Set newXMLAttribute = diagramSave.createNode(NODE_ATTRIBUTE, "type", Empty)
        newXMLAttribute.Text = attributeIterator.attributeType
        Call attributeNode.attributes.setNamedItem(newXMLAttribute)
        'value
        Set newXMLAttribute = diagramSave.createNode(NODE_ATTRIBUTE, "value", Empty)
        newXMLAttribute.Text = attributeIterator.attributeValue
        Call attributeNode.attributes.setNamedItem(newXMLAttribute)
        'visibility
        Set newXMLAttribute = diagramSave.createNode(NODE_ATTRIBUTE, "visibility", Empty)
        newXMLAttribute.Text = attributeIterator.visibility
        Call attributeNode.attributes.setNamedItem(newXMLAttribute)
        'parameters
        For Each parameterIterator In attributeIterator.parameters
            'create new node
            Set parameterNode = diagramSave.createNode(NODE_ELEMENT, "parameter", Empty)
            'name
            Set newXMLAttribute = diagramSave.createNode(NODE_ATTRIBUTE, "name", Empty)
            newXMLAttribute.Text = parameterIterator.name
            Call parameterNode.attributes.setNamedItem(newXMLAttribute)
            'kind
            Set newXMLAttribute = diagramSave.createNode(NODE_ATTRIBUTE, "kind", Empty)
            newXMLAttribute.Text = parameterIterator.kind
            Call parameterNode.attributes.setNamedItem(newXMLAttribute)
            'type
            Set newXMLAttribute = diagramSave.createNode(NODE_ATTRIBUTE, "type", Empty)
            newXMLAttribute.Text = parameterIterator.parameterType
            Call parameterNode.attributes.setNamedItem(newXMLAttribute)
            'add to attribute node
            Call attributeNode.appendChild(parameterNode)
        Next parameterIterator
        'add to parent
        Call newXMLNode.appendChild(attributeNode)
    Next attributeIterator
    'return constructed node
    Set ParseNode = newXMLNode
End Function

Private Sub BuildDiagram(buildOn As IXMLDOMNode, buildWith As CNode, depth As Integer, maxDepth As Integer)
'Takes the buildOn XML node, and adds a buildWidth (parsed into XML) to it as a child node. Then iterates
'through buildWith's children, adding them to the last child, which should be the parsed buildWidth.

    On Error Resume Next

    'Debug.Print "Building with " & buildWith.nodeName
    'adds buildWith to buildOn
    Dim childIterator As CNode
    
    'now add to diagram
    Call buildOn.appendChild(ParseNode(buildWith))
    'now do children of this node if we're not too deep
    If depth + 1 <= maxDepth Then
        For Each childIterator In buildWith.childNodes
            'Debug.Print "Adding " & childIterator.nodeName & " to " & buildOn.lastChild.attributes.getNamedItem("name").Text
            Call BuildDiagram(buildOn.lastChild, childIterator, depth + 1, maxDepth)
        Next childIterator
    End If
End Sub
