Attribute VB_Name = "modSpatialSound"
'handles DirectX playing of spatial sound around user, corresponding to joystick movement.
Option Explicit

Private dxObject As New DirectX8
Private ds As DirectSound8
Private loader As DirectMusicLoader8
Private soundPerformance As DirectMusicPerformance8
Private segment As DirectMusicSegment8
Private segmentState As DirectMusicSegmentState8
Private audioPath As DirectMusicAudioPath8
Private Const soundChannel As Long = 1
Private Const SND_DISTANCE_SCALING_FACTOR As Single = 1
Private Const PLAY_ONCE As Integer = 0
Private playing As Boolean
Private mVolume As Long ' the volume of the system, ranging from -1800 to + 200

Public Function getVolume() As Long
    On Error Resume Next
    'convert actual volume (-1800 to 200) to useful value (0 to 10)
    getVolume = ((mVolume - 200) / 200) + 10
    'Debug.Print "getVolume = " & getVolume
End Function

Public Sub letVolume(newVolume As Long)
    On Error Resume Next
    'convert useful value (0 to 10) to actual volume (-1800 to 200)
    If newVolume < 0 Then newVolume = 0
    If newVolume > 10 Then newVolume = 10
    mVolume = (newVolume - 10) * 200 + 200
    Call StopMidi 'stop so we restart, so the new volume takes effect
    'Debug.Print "letVolume= " & mVolume
End Sub

Public Sub PlaySpatialSound(direction As Integer)
    On Error Resume Next
    If Not frmMain.mnuOptionsSounds.Checked Then Exit Sub
    If playing Then
    Else
        Call PlayLocationAroundUser(direction * 22.5, App.path & "\sounds\spatial.wav")
    End If
End Sub

Public Sub StopSpatialSound()
    On Error Resume Next
    If playing Then
        Call StopMidi
    End If
End Sub

Public Sub PlayMapSound(x As Single, y As Single)
    On Error Resume Next
    If Not frmMain.mnuOptionsSounds.Checked Then Exit Sub
    If playing Then
    Else
        Call PlayLocationAsCoordinates(x, y)
    End If
End Sub

Public Sub StopMapSound()
    On Error Resume Next
    If playing Then
        Call StopMidi
    End If
End Sub


Public Sub Initialize()
' sets up all the DirectX sound objects and the load paths.
' Must be called at beginning of application.
    On Error GoTo errorHandler
    Dim audioParameters As DMUS_AUDIOPARAMS
    
    Set ds = dxObject.DirectSoundCreate(vbNullString)
    Set loader = dxObject.DirectMusicLoaderCreate
    Call loader.SetSearchDirectory(App.path & "\sounds")
    Set soundPerformance = dxObject.DirectMusicPerformanceCreate
    Call soundPerformance.InitAudio(frmMain.hWnd, DMUS_AUDIOF_ALL, audioParameters)
    Set audioPath = soundPerformance.CreateStandardAudioPath(DMUS_APATH_DYNAMIC_3D, 64, True)
    mVolume = 10000 'play loud by default
    Exit Sub
errorHandler:
    Resume Next
    Exit Sub
End Sub

Private Sub StopMidi()
' terminates the playing of midi files.  Must be updated if new segments are introduced
    On Error Resume Next
    If Not (segment Is Nothing) Then
        Call soundPerformance.StopEx(segment, 0, 0)
        Call segment.Unload(audioPath)
        Set segment = Nothing
    End If
    playing = False
End Sub

Public Sub CleanUp()
' call when application is terminating.  Kills objects and stops sounds
    On Error Resume Next
    Call StopMidi
    Set audioPath = Nothing
    Set segmentState = Nothing
    Set segment = Nothing
    Set soundPerformance = Nothing
    Set loader = Nothing
    Set ds = Nothing
    Set dxObject = Nothing
End Sub

Public Sub PlayMidi(midiToPlay As String, repeats As Long)
    ' Now play the music
    On Error GoTo errorHandler
    Dim buffer3D As DirectSound3DBuffer8
    
    Call StopMidi
    Set segment = loader.LoadSegment(midiToPlay)
    If InStr(1, midiToPlay, ".mid") > 0 Then ' check it is a midi, now wav file
        Call segment.SetStandardMidiFile
    End If
    Call segment.Download(audioPath)
    Call segment.SetRepeats(repeats)
    Set buffer3D = audioPath.GetObjectinPath(DMUS_PCHANNEL_ALL, DMUS_PATH_BUFFER, 0, GUID_ALL, 0, IID_DirectSound3DBuffer)
    Call buffer3D.SetPosition(0, 0, 0, DS3D_IMMEDIATE)
    Call soundPerformance.SetMasterVolume(mVolume)
    Set segmentState = soundPerformance.PlaySegmentEx(segment, DMUS_SEGF_DEFAULT, 0, Nothing, audioPath)
    Set buffer3D = Nothing
    Exit Sub
errorHandler:
    Resume Next
End Sub

Private Sub PlayLocationAroundUser(direction As Integer, soundFile As String)
'plays the item location sound in the direction indicated by direction and ...
    On Error Resume Next
    Dim x As Single
    Dim y As Single
    Dim buffer3D As DirectSound3DBuffer8
    
    If Not (segment Is Nothing) Then
        Call segment.Unload(audioPath)
        Set segment = Nothing
    End If
    Set segment = loader.LoadSegment(soundFile)
    'Call segment.SetStandardMidiFile
    Call segment.Download(audioPath)
    Call segment.SetRepeats(10000)
    Set buffer3D = audioPath.GetObjectinPath(DMUS_PCHANNEL_ALL, DMUS_PATH_BUFFER, 0, GUID_ALL, 0, IID_DirectSound3DBuffer)
    x = Sin(CSng(direction) * PI / 180)
    y = Cos(CSng(direction) * PI / 180)
    If x < 0.01 And x > -0.01 Then x = 0
    If y < 0.01 And y > -0.01 Then y = 0
    'frmDebugJoystick.txtX.Text = X
    'frmDebugJoystick.txtY.Text = Y
    Call buffer3D.SetPosition(x, 0, y, DS3D_IMMEDIATE)
    Call soundPerformance.SetMasterVolume(mVolume)
    Set segmentState = soundPerformance.PlaySegmentEx(segment, DMUS_SEGF_DEFAULT, 0, Nothing, audioPath)
    Set buffer3D = Nothing
    playing = True
End Sub

Private Sub PlayLocationAsCoordinates(x As Single, y As Single)
'plays the item location sound in the location indicated by x and y
    On Error Resume Next
    Dim buffer3D As DirectSound3DBuffer8
    'Debug.Print "playing " & x & "," & y
            
    If Not (segment Is Nothing) Then
        Call segment.Unload(audioPath)
        Set segment = Nothing
    End If
    Set segment = loader.LoadSegment(App.path & "\sounds\spatial.wav")
    'Call segment.SetStandardMidiFile
    Call segment.Download(audioPath)
    Call segment.SetRepeats(10000)
    Set buffer3D = audioPath.GetObjectinPath(DMUS_PCHANNEL_ALL, DMUS_PATH_BUFFER, 0, GUID_ALL, 0, IID_DirectSound3DBuffer)
    Call buffer3D.SetPosition(x, 0, y, DS3D_IMMEDIATE)
    Call soundPerformance.SetMasterVolume(mVolume)
    Set segmentState = soundPerformance.PlaySegmentEx(segment, DMUS_SEGF_DEFAULT, 0, Nothing, audioPath)
    Set buffer3D = Nothing
    playing = True
End Sub

